function customStartup()
{
	makeTranslucentBackground('header_span_header', '/blojsom_resources/images/span_header.png');
	makeTranslucentBackground('header_span_body', '/blojsom_resources/images/span_body.png');
	makeTranslucentBackground('header_span_footer', '/blojsom_resources/images/span_footer.png');

	makeTranslucentBackground('calendar_box_header', '/blojsom_resources/images/side_box_header.png');
	makeTranslucentBackground('calendar_box_body', '/blojsom_resources/images/side_box_body.png');
	makeTranslucentBackground('calendar_box_footer', '/blojsom_resources/images/side_box_footer.png');

	makeTranslucentBackground('search_box_header', '/blojsom_resources/images/side_box_header.png');
	makeTranslucentBackground('search_box_body', '/blojsom_resources/images/side_box_body.png');
	makeTranslucentBackground('search_box_footer', '/blojsom_resources/images/side_box_footer.png');
	
	makeTranslucentBackground('links_box_header', '/blojsom_resources/images/side_box_header.png');
	makeTranslucentBackground('links_box_body', '/blojsom_resources/images/side_box_body.png');
	makeTranslucentBackground('links_box_footer', '/blojsom_resources/images/side_box_footer.png');
	
	makeTranslucentBackground('blogs_box_header', '/blojsom_resources/images/main_box_header.png');
	makeTranslucentBackground('blogs_box_body', '/blojsom_resources/images/main_box_body.png');
	makeTranslucentBackground('blogs_box_footer', '/blojsom_resources/images/main_box_footer.png');
	
	makeTranslucentBackground('login_dialog_header', '/blojsom_resources/images/dialog_400_header.png');
	makeTranslucentBackground('login_dialog_body', '/blojsom_resources/images/dialog_400_body.png');
	makeTranslucentBackground('login_dialog_footer', '/blojsom_resources/images/dialog_400_footer.png');
	
	makeTranslucentBackground('settings_dialog_header', '/blojsom_resources/images/dialog_500_header.png');
	makeTranslucentBackground('settings_dialog_body', '/blojsom_resources/images/dialog_500_body.png');
	makeTranslucentBackground('settings_dialog_footer', '/blojsom_resources/images/dialog_500_footer.png');
	
	makeTranslucentBackground('entry_dialog_header', '/blojsom_resources/images/dialog_500_header.png');
	makeTranslucentBackground('entry_dialog_body', '/blojsom_resources/images/dialog_500_body.png');
	makeTranslucentBackground('entry_dialog_footer', '/blojsom_resources/images/dialog_500_footer.png');
	
	makeTranslucentBackground('category_dialog_header', '/blojsom_resources/images/dialog_500_header.png');
	makeTranslucentBackground('category_dialog_body', '/blojsom_resources/images/dialog_500_body.png');
	makeTranslucentBackground('category_dialog_footer', '/blojsom_resources/images/dialog_500_footer.png');
}
